% step_on_rake_JC_revision.m
% replicates sims' stepping on a rake model
% this script sets up many different cases . 
% you can just use test_function and f_step_on_rake for one use

% NOTE all programs use sims notation, not paper notation. Changing 
% seemed likely to introduce bugs. 
% paper / programs (sims)
% i is r
% phi_pi is theta
% phi_c is phi
% r is is rho
% i is r 
% y is a 
% rho is bet 
% s is tau
% kappa is delta
% z is y

close all
clear all
T = 100; %15; % length of simulations
dt = 0.01;
printfigs = 1; % print figures? Will overwrite eps files 

% which cases to do -- zero lets you work on one case
sims = 1; % do sims parameters
case_2 = 1; % no Taylor rule zero  psi
shortdebt = 1; % do both sims and case 2 with short term debt
less_sticky = 1; % look at less price stickiness. 

% ---------------------
% Replicate sims
% --------------------

if sims;
    gam = 0.5;
    thet = 0.4;
    phi = 0.75;
    sig = 2.0;
    rhobar = 0.05;
    rhohat = 0.05; % In ms programs, set to zero to replicate sims typo. No longer implemented, 
    taubar = 0.10;
    bet = 0.10;
    delt = 0.20;
    omeg = 1;
    psi = 2;
    longdebt = 1;
    em = 0.001;
    etau = 0; % etau = -0.001 replicates sims fig 4 exactly
    
    %[eigs,t,rt, pit,at, taut,bt,lamt,ct,cdott,pt,rhot,Bt,...
    %    ta,rta, pita,ata, tauta,bta,lamta,cta,cdotta,pta,rhota,Bta] =...
     %   f_step_on_rake(gam,thet,phi,sig,rhobar,rhohat,taubar,bet,delt,omeg,psi,...
     %   longdebt,T,dt,em,etau);
    
      [eigs,t,rt, pit,at, taut,bt,lamt,ct,cdott,pt,rhot,Bt,...
        ta,rta, pita,ata, tauta,bta,lamta,cta,cdotta,pta,rhota,Bta] =...
        f_step_on_rake_revision(gam,thet,phi,sig,rhobar,rhohat,taubar,bet,delt,omeg,psi,...
        longdebt,T,dt,em,etau);
    
    script_sims_figure_revision; %script to make sims figure as exactly as possible
    
    if printfigs;
        set(gcf, 'PaperPosition', [0.25 2.5 6.0 8.0]);
        % print sims figure
        print -depsc2 sims_fig_3.eps
        savefig ('sims_fig_3');
    end;

    script_sims_rates_graph_revision; % makes my graph of all rates for sims case
    text(1,0.6,'Nominal rate i','color',colorvec(1,:),'fontsize',16)
    text(3,-0.09,'Inflation \pi','color',colorvec(2,:),'fontsize',16)
    axis([-1 10 -0.2 1.03]); 
    if printfigs;
        print -depsc2 sims_r.eps
        savefig('sims_r');
    end;

    % script_sims_other_graph; % makes my graph of other quantities. 
    
    script_sims_rates_expected_graph_revision; %makes graph of rates when shock expected
    text(-0.2,0.6,'Nominal rate i','color',colorvec(1,:),'horizontalalignment','r','fontsize',16)
    text(2,0.4,'Inflation \pi','color',colorvec(2,:),'fontsize',16)
     axis([-10 10 -0.1 1.2]);
     
    if printfigs;
        print -depsc2 sims_r_expected.eps
        savefig('sims_r_expected')
    end;

end;
  
% -------------------------------
% no habits, taylor rule, or fiscal policy
% ------------------------------
 
if case_2;
    gam = 0;
    thet = 0;
    phi = 0;
    sig = 2.0;
    rhobar = 0.05;
    rhohat = 0.05; % set to zero to replicate sims typo
    taubar = 0.10;
    bet = 0.10;
    delt = 0.20;
    omeg = 0;
    psi = 0; % Note: I have verified that very small psi produces the same result
    longdebt = 1;
    em = 0.001;
    etau = 0;
    
    [eigs,t,rt, pit,at, taut,bt,lamt,ct,cdott,pt,rhot,Bt,...
        ta,rta, pita,ata, tauta,bta,lamta,cta,cdotta,pta,rhota,Bta] =...
        f_step_on_rake_revision(gam,thet,phi,sig,rhobar,rhohat,taubar,bet,delt,omeg,psi,...
        longdebt,T,dt,em,etau);
    
    %script_sims_figure; %script to make sims figure as exactly as possible
    
    script_sims_rates_graph_revision; % makes my graph of all rates for sims case
    text(1,0.6,'Nominal rate i','color',colorvec(1,:),'fontsize',16)
    text(5,-1.2,'Inflation \pi','color',colorvec(2,:),'fontsize',16)
    axis([-1 10 -5.8 1.2]); 
    if printfigs;
        print -depsc2 case_2_r.eps
        savefig('case_2_r')
        
    end;
   
    %script_sims_other_graph; % makes my graph of other quantities. 
    
    script_sims_rates_expected_graph_revision; %makes graph of rates when shock expected
    text(-0.2,0.8,'Nominal rate i','color',colorvec(1,:),'horizontalalignment','r','fontsize',16)
    text(2,0.5,'Inflation \pi','color',colorvec(2,:),'fontsize',16)
    axis([-10 10 -0.1 1.1]);
    if printfigs;
        print -depsc2 case_2_r_expected.eps
        savefig('case_2_r_expected')
    end;
end;

% ----------------------
% short term debt
% ----------------------

if shortdebt;
    % first sims with short debt
    gam = 0.5;
    thet = 0.4;
    phi = 0.75;
    sig = 2.0;
    rhobar = 0.05;
    rhohat = 0.05; % set to zero to replicate sims typo
    taubar = 0.10;
    bet = 0.10;
    delt = 0.20;
    omeg = 1;
    psi = 2;
    longdebt = 0; % this is the only difference from above. 
    em = 0.001;
    etau = 0; % etau = -0.001 replicates sims fig 4 exactly
    
    [eigs,t,rt, pit,at, taut,bt,lamt,ct,cdott,pt,rhot,Bt,...
        ta,rta, pita,ata, tauta,bta,lamta,cta,cdotta,pta,rhota,Bta] =...
        f_step_on_rake_revision(gam,thet,phi,sig,rhobar,rhohat,taubar,bet,delt,omeg,psi,...
        longdebt,T,dt,em,etau);
    
    script_sims_rates_graph_revision; % makes my graph of all rates for sims case
    text(1,0.8,'Nominal rate i','color',colorvec(1,:),'fontsize',16)
    text(1,0.2,'Inflation \pi','color',colorvec(2,:),'fontsize',16)
    axis([-1 10 -0.2 1.03]); 
   
    if printfigs;
        print -depsc2 sims_r_short.eps
        savefig('sims_r_short');
    end;
    
    script_sims_rates_expected_graph_revision; %makes graph of rates when shock expected
    text(-0.2,0.8,'Nominal rate i','color',colorvec(1,:),'horizontalalignment','r','fontsize',16)
    text(2,0.3,'Inflation \pi','color',colorvec(2,:),'fontsize',16)
     axis([-10 10 -0.1 1.2]);
   
    if printfigs;
        print -depsc2 sims_r_short_expected.eps
        savefig('sims_r_short_expected')
    end;
    %
 
    % no habits, taylor rule, or fiscal policy with short debt 
 
    gam = 0;
    thet = 0;
    phi = 0;
    sig = 2.0;
    rhobar = 0.05;
    rhohat = 0.05; % set to zero to replicate sims typo
    taubar = 0.10;
    bet = 0.10;
    delt = 0.20;
    omeg = 0;
    psi = 0;
    longdebt = 0; % this is the only difference from case_2 above 
    em = 0.001;
    etau = 0;
    
    [eigs,t,rt, pit,at, taut,bt,lamt,ct,cdott,pt,rhot,Bt,...
        ta,rta, pita,ata, tauta,bta,lamta,cta,cdotta,pta,rhota,Bta] =...
        f_step_on_rake_revision(gam,thet,phi,sig,rhobar,rhohat,taubar,bet,delt,omeg,psi,...
        longdebt,T,dt,em,etau);
    
    
    script_sims_rates_graph_revision; % makes my graph of all rates for sims case
    text(1,0.95,'Nominal rate i','color',colorvec(1,:),'fontsize',16)
    text(1,1.05,'Inflation \pi','color',colorvec(2,:),'fontsize',16)
    axis([-1 10 -0.1 1.1]); 
   
    if printfigs;
        print -depsc case_2_r_shortdebt.eps
        savefig('case_2_r_shortdebt')
    end;
    
    script_sims_rates_expected_graph_revision; %makes graph of rates when shock expected
     text(-0.2,0.8,'Nominal rate i','color',colorvec(1,:),'horizontalalignment','r','fontsize',16)
    text(2,0.5,'Inflation \pi','color',colorvec(2,:),'fontsize',16)
     axis([-10 10 -0.1 1.1]);
   
    if printfigs;
        print -depsc2 case_2_r_expected_shortdebt.eps
        savefig('case_2_r_expected_shortdebt')
    end;

end;


% ----------------------
% less sticky
% ----------------------

if less_sticky;
   
    gam = 0;
    thet = 0;
    phi = 0;
    sig = 2.0;
    rhobar = 0.05;
    rhohat = 0.05; % set to zero to replicate sims typo
    taubar = 0.10;
    bet = 0.10;
    deltv = [0.20  1  100 0.02 0.001];
    omeg = 0;
    psi = 0;
    longdebt = 1; % this is the only difference from case_2 above 
    em = 0.001;
    etau = 0;
    
      
    for indx = 1:size(deltv,2); 
       [eigs,t,rt, pit,at, taut,bt,lamt,ct,cdott,pt,rhot,Bt,...
        ta,rta, pita,ata, tauta,bta,lamta,cta,cdotta,pta,rhota,Bta] =...
        f_step_on_rake_revision(gam,thet,phi,sig,rhobar,rhohat,taubar,bet,deltv(indx),omeg,psi,...
        longdebt,T,dt,em,etau);
       pitv(:,indx) = pit; 
       ptv(:,indx) = pt; 
    end;
    
   
    figure;
    plot([-1;0;t],[[0;0]*ones(1,size(pitv,2)); pitv]/em, 'linewidth',2);
    hold on
    plot([-1;0;t],[0;0; rt ]/em,'linewidth',2);
    colorvec = get(gcf,'defaultaxescolororder');
    plot(t(1:1/dt:end),[rt(1:1/dt:end)]/em,'v','color',colorvec(1,:),'linewidth',2);
    %plot(t(1:1/dt:end),[pit(1:1/dt:end)]/em,'o','color',colorvec(4,:),'linewidth',2);
    plot(t,0*t,'-k');
    ylabel('%');
    text(-0.2,1,'i','color',colorvec(1,:),'horizontalalignment','right','fontsize',16);
    for indx = 1:size(pitv,2);
        text(-.2,pitv(1,indx)/em,['\kappa =' num2str(deltv(indx),1)],'color',colorvec(indx+1,:),...
            'horizontalalignment','right','fontsize',16,'backgroundcolor','w');
    end;
    axis([-1 5 -inf 2]);
    
    if printfigs;
        print -depsc2 less_sticky_inflation.eps
        savefig('less_sticky_inflation');
    end;
    
    figure;
    plot([-1; 0; t], [[0;0]*ones(1,size(ptv,2)+1); [rt ptv]]/em,'linewidth',2);
    hold on
    colorvec = get(gcf,'defaultaxescolororder');
    plot(t(1:1/dt:end),[rt(1:1/dt:end)]/em,'v','color',colorvec(1,:),'linewidth',2);
    plot([-1; 0; t(1:1/dt:end)], [0;0;ptv(1:1/dt:end,1)]/em, 'o','linewidth',2);
    %plot(t(1:1/dt:end),[pit(1:1/dt:end)]/em,'o','color',colorvec(4,:),'linewidth',2);
    plot(t,0*t,'-k');
    ylabel('Percent');
    text(-0.3,1,'i','color',colorvec(1,:),'horizontalalignment','right','fontsize',16);
    text(5-1,ptv((5-1)/dt,1)/em+1,['\kappa =' num2str(deltv(1),'%4.2f')],'color',colorvec(1+1,:),...
            'horizontalalignment','left','fontsize',16,'backgroundcolor','w');
    text(5-2,ptv((5-2)/dt,2)/em+1,['\kappa =' num2str(deltv(2),'%4.0f')],'color',colorvec(2+1,:),...
            'horizontalalignment','left','fontsize',16,'backgroundcolor','w');
   % text(5-3,ptv((5-3)/dt,3)/em,['\kappa =' num2str(deltv(3),'%4.0f')],'color',colorvec(3+1,:),...
   %         'horizontalalignment','left','fontsize',16,'backgroundcolor','w');
    text(0.5,ptv((0.5)/dt,3)/em-1,['\kappa =' num2str(deltv(3),'%4.0f')],'color',colorvec(3+1,:),...
            'horizontalalignment','left','fontsize',16,'backgroundcolor','w');
    text(7,ptv((7)/dt,4)/em+1,['\kappa =' num2str(deltv(4),'%4.3f')],'color',colorvec(4+1,:),...
            'horizontalalignment','left','fontsize',16,'backgroundcolor','w');
   text(8,ptv((8)/dt,5)/em-1,['\kappa =' num2str(deltv(5),'%4.3f')],'color',colorvec(5+1,:),...
            'horizontalalignment','left','verticalalignment','top','fontsize',16,'backgroundcolor','w');
  
   xlabel('Time');
    
    %for indx = 1:size(ptv,2);
    %    text(4-indx+1,ptv((4-indx+1)/dt,indx)/em,['\kappa =' num2str(deltv(indx),'%4.0f')],'color',colorvec(indx+1,:),...
    %        'horizontalalignment','left','fontsize',16,'backgroundcolor','w');
    %end;
    axis([-1 10 -22 2]);
    
    if printfigs;
        print -depsc2 less_sticky_price.eps
        savefig('less_sticky_price');
    end;
   
   
    % *************** 
    % less sticky in the full sims model 
    % ***************
    
    
    gam = 0.5;
    thet = 0.4;
    phi = 0.75;
    sig = 2.0;
    rhobar = 0.05;
    rhohat = 0.05; % set to zero to replicate sims typo
    taubar = 0.10;
    bet = 0.10; 
    deltv = [0.2 200 2 0.05];
    omeg = 1;
    psi = 2;
    longdebt = 1;  
    em = 0.001;
    etau = 0;
    
    clear pitv ptv
      
    for indx = 1:size(deltv,2); 
       [eigs,t,rt, pit,at, taut,bt,lamt,ct,cdott,pt,rhot,Bt,...
        ta,rta, pita,ata, tauta,bta,lamta,cta,cdotta,pta,rhota,Bta] =...
        f_step_on_rake_revision(gam,thet,phi,sig,rhobar,rhohat,taubar,bet,deltv(indx),omeg,psi,...
        longdebt,T,dt,em,etau);
       pitv(:,indx) = pit; 
       ptv(:,indx) = pt; 
       disp(''); 
       disp('delta'); disp(deltv(indx)); 
       disp('eigenvalues'); disp(eigs');
       disp('number positive'); disp(sum(real(eigs)>0));
    end;
    
   
    figure;
    plot([-1;0;t],[[0;0]*ones(1,size(pitv,2)); pitv]/em, 'linewidth',2);
    hold on
    plot([-1;0;t],[0;0; rt ]/em,'linewidth',2);
    colorvec = get(gcf,'defaultaxescolororder');
    plot(t(1:1/dt:end),[rt(1:1/dt:end)]/em,'v','color',colorvec(1,:),'linewidth',2);
    %plot(t(1:1/dt:end),[pit(1:1/dt:end)]/em,'o','color',colorvec(4,:),'linewidth',2);
    plot(t,0*t,'-k');
    ylabel('%');
   % text(-0.2,1,'i','color',colorvec(1,:),'horizontalalignment','right','fontsize',16);
   % for indx = 1:size(pitv,2);
   %     text(-.2,pitv(1,indx)/em,['\kappa =' num2str(deltv(indx),1)],'color',colorvec(indx+1,:),...
   %         'horizontalalignment','right','fontsize',16,'backgroundcolor','w');
   % end;
   % axis([-1 5 -inf 2]);
    
    if printfigs;
        print -depsc2 less_sticky_inflation_full.eps
        savefig('less_sticky_inflation_full');
    end;
    
    figure;
    plot([-1; 0; t], [[0;0]*ones(1,size(ptv,2)+1); [rt ptv]]/em,'linewidth',2);
    hold on
    colorvec = get(gcf,'defaultaxescolororder');
    plot(t(1:1/dt:end),[rt(1:1/dt:end)]/em,'v','color',colorvec(1,:),'linewidth',2);
    plot([-1; 0; t(1:1/dt:end)], [0;0;ptv(1:1/dt:end,1)]/em, 'o','linewidth',2);
    %plot(t(1:1/dt:end),[pit(1:1/dt:end)]/em,'o','color',colorvec(4,:),'linewidth',2);
    plot(t,0*t,'-k');
    ylabel('Percent');
    text(2.5,0.5,'i_t','color',colorvec(1,:),'fontsize',16);
    text(7,-0.7, 'p_t, \kappa = 0.05, 0.2, 2, 200','color',colorvec(2,:), 'fontsize',16);
   xlabel('Time');
    
    axis([-1 30 -inf 1.1]);
    
    if printfigs;
        print -depsc2 less_sticky_price_full.eps
        savefig('less_sticky_price_full')
    end;
   
    
    
    
    
end;

 stop 

 % What follows is an exploration of solving the full 
 % nonlinear system. Search for delta zero that keeps the 
 % nonlinear system from exploding. I verify that 
 % linear dynamics have the same slope for small t, but 
 % nonlienar dyanmics then explode away about year 3-4. 
 % however, teh resulting deltas were smal changes -- small changes at zero
 % 
 % stop the nonlinear explosions -- so didn't really affect dynamics that
 % much. This ends up being a confirmation that linear is pretty good 
 % for this purpose. 
%Show linear vs nonlinear system starting at x0; 

[rxt,pixt,axt,tauxt,bxt,lamxt,cxt,cdotxt,rhoxt,pxt,Bxt] =...
      f_sims_simulation(x0,gam,rhobar,thet,phi,dt,bet,delt,omeg,taubar,psi,sig,T); 

figure;
plot(t,100*[rxt-rhobar axt-rhobar rhoxt-rhobar pixt],'linewidth',2);
hold on; plot(t,0*t,'-k');
plot(t,100*[rt at rhot pit],'--','linewidth',2);

ylabel('%');
legend('i', 'a' ,'r' ,'\pi');
axis([-.1 T -inf inf]); 

figure;
subplot(2,2,1);
plot(t,[Bxt-b*rhobar Bt],'linewidth',2);
hold on; 
plot(t,0*t,'-k');
ylabel('B')
axis([-.1 T -inf inf]);

subplot(2,2,2);
plot(t,100*[cxt ct],'linewidth',2);
hold on; 
plot(t,0*t,'-k');
ylabel('c, %');
axis([-.1 T -inf inf]);

subplot(2,2,3)
plot(t,[bxt-b bt],'linewidth',2);
hold on; 
plot(t,0*t,'-k');
ylabel('b');
axis([-.1 T -inf inf]);

subplot(2,2,4);
plot(t,[lamxt-1 lamt],'linewidth',2);
hold on; 
plot(t,0*t,'-k');
ylabel('lam');
axis([-.1 T -inf inf]);


% do search for deltas that work for the nonlinear system


objective = @(deltas) f_obj(deltas,em,gam,rhobar,thet,phi,dt,bet,delt,omeg,taubar,psi,sig,T);

deltas_opt = fminsearch(objective,x0([2 3 6 8]));

       %r    pi      a         tau       b            lam        c    cdot    , devaiations from st st. 
x0 = [ em deltas_opt(1) deltas_opt(2)  0  -b/rhobar*deltas_opt(2) deltas_opt(3)  0  deltas_opt(4)]'; 

[rxt,pixt,axt,tauxt,bxt,lamxt,cxt,cdotxt,rhoxt,pxt,Bxt] =...
      f_sims_simulation(x0,gam,rhobar,thet,phi,dt,bet,delt,omeg,taubar,psi,sig,T); 


figure;
plot(t,100*[rxt-rhobar axt-rhobar rhoxt-rhobar pixt],'linewidth',2);
hold on; plot(t,0*t,'-k');
plot(t,100*[rt at rhot pit],'--','linewidth',2);

ylabel('%');
legend('i', 'a' ,'r' ,'\pi');
axis([-.1 T -inf inf]); 

figure;
subplot(2,2,1);
plot(t,[Bxt-b*rhobar Bt],'linewidth',2);
hold on; 
plot(t,0*t,'-k');
ylabel('B')
axis([-.1 T -inf inf]);

subplot(2,2,2);
plot(t,100*[cxt ct],'linewidth',2);
hold on; 
plot(t,0*t,'-k');
ylabel('c, %');
axis([-.1 T -inf inf]);

subplot(2,2,3)
plot(t,[bxt-b bt],'linewidth',2);
hold on; 
plot(t,0*t,'-k');
ylabel('b');
axis([-.1 T -inf inf]);

subplot(2,2,4);
plot(t,[lamxt-1 lamt],'linewidth',2);
hold on; 
plot(t,0*t,'-k');
ylabel('lam');
axis([-.1 T -inf inf]);


    
if case_2; % manual parameters
    
    gam = 0.0; % 0.5 for sims;
    thet = 0.0 ; %0.4 for sims;
    phi = 0.0 ; % 0.75 for sims;
    sig = 2.0;
    rhobar = 0.05;
    rhohat = 0.05; % set to zero to replicate sims typo
    taubar = 0.10;
    bet = 0.10;
    delt = 20%0.20;
    omeg = 0; % 1 for sims;
    psi = 0.00; % 2 for sims
    longdebt = 1; % flag, 1 for short debt, 0 for long debt. 
    % just deletes the jump in b. 
    
end;

    
if case_manual; % manual parameters
    
    gam = 0.001; % 0.5 for sims;
    thet = 0.0 ; %0.4 for sims;
    phi = 0.0 ; % 0.75 for sims;
    sig = 2.0;
    rhobar = 0.05;
    rhohat = 0.05; % set to zero to replicate sims typo
    taubar = 0.10;
    bet = 0.10;
    delt = 0.20;
    omeg = 0; % 1 for sims;
    psi = 0.00; % 2 for sims
    longdebt = 1; % flag, 1 for short debt, 0 for long debt. 
    % just deletes the jump in b. 
    
end;


